#ifndef PB_KERNEL_H
#define PB_KERNEL_H
#define REG(x) (*((unsigned short*)(x)))

namespace poobear
{
   extern int* registers;
   extern int& returnFlag;
	const int SCREEN_WIDTH = 240;
	const int SCREEN_HEIGHT = 160;

   class Module
   {
      public:
         Module(int p = 0):
            itsPriority(p) {}
         virtual ~Module() {}

         virtual void insert() = 0;
         virtual void clean() = 0;
         virtual void step() = 0;
         int getPriority() const { return itsPriority; }

      protected:
         int itsPriority;
   };
   
   class Kernel
   {
      public:
         static Kernel* getInstance(int size = 20);

         int run();
         void addModule(Module*);
         Module* removeModule(Module*);

         void flush();

         // palette operations
         void setSpritePalette(unsigned short*);
         void setBackgroundPalette(unsigned short*);


         // utilities
         void softReset();
         void DMACopyAsync(void* src, void* dest, int length, int channel);
         void DMACopySync(void* src, void* dest, int length, int channel);

      private:
         Kernel(int);

         static Kernel* instance;

         int itsSize;
         Module** modules;
   };
};
#endif

