#ifndef PB_KEYS_H
#define PB_KEYS_H
#include "kernel.h"

namespace poobear
{
   // enum of all the keys on a GBA
	enum KEY
	{
		KEY_A,
		KEY_B,
		KEY_SELECT,
		KEY_START,
		KEY_RIGHT,
		KEY_LEFT,
		KEY_UP,
		KEY_DOWN,
		KEY_RTRIGGER,
		KEY_LTRIGGER
	};

   /* 
    * A generic keylistener. User should 
    * subclass from this class and override the
    * corresponding methods.
    */
	class KeyListener : public Module
	{
		public:
			KeyListener(int p = 2):
				Module(p),
				prevState(0) {}

         /*
          * Inherited from Module and implemented
          * Do not overwrite.
          */
			void insert() {};
			void step();
			void clean() {};

         /*
          * Override this functions to define
          * the behavior of the game when a key 
          * is pressed, release, or held down
          */
			virtual void keyPressed(KEY) {};
			virtual void keyReleased(KEY) {};
			virtual void keyDown(KEY) {};

		private:
			int state; 
			int prevState;
	};
};
#endif


